﻿(function ($) {

    var ctor = function () { };

    var inherits = function (parent, protoProps, staticProps) {
        var child;

        if (protoProps && protoProps.hasOwnProperty('constructor')) {
            child = protoProps.constructor;
        } else {
            child = function () { parent.apply(this, arguments); };
        }

        $.extend(child, parent);

        ctor.prototype = parent.prototype;
        child.prototype = new ctor();

        if (protoProps) $.extend(child.prototype, protoProps);

        if (staticProps) $.extend(child, staticProps);

        child.prototype.constructor = child;

        child.__super__ = parent.prototype;

        return child;
    };

    var extend = function (protoProps, classProps) {
        var child = inherits(this, protoProps, classProps);
        child.extend = this.extend;
        return child;
    };

    /*
    *Start the main function of this project
    * 
    */

    /*
    * Declare CountryIcon class
    */
    var direction = {
        startToEnd: 0,
        endToStart: 1
    }
    var Icon = function () {
        this.el = null;
        this.name = "";
        this.state = {
            index: 0,
            direction: direction.startToEnd,
            suspend: true
        };
        this.animationQueue = $.Deferred();
        this.position = {
            x: 0,
            y: 0
        };
        this.container = null;
        this.dfds = [];// store the function promises to ensure that all images can be loaded safetly
        //For interval
        this.duration = 250;
        this.interVal = 2000;


        this.init.apply(this, arguments);

        this.suspend = function () {
            this.state.suspend = true;
        }

        this.restart = function () {
            this.state.suspend = false;            
        }
    },

    CountryIcon = inherits(Icon, {
        init: function (options) {            
            var that = this;
            this.description = "" || $(options.description);
            this.name = options.name;
            this.el = $("<div class='country'>");            
            this.position = options.position || this.position;
            this.el.css({"left": this.position.x, "top": this.position.y });
            this.personIcon = new PersonIcon(options.person);
            this.personIcon.container = this.el;
            this.duration = 150;            
            //Define the boundary of the animation
            this.boundary = { high: this.position.y + 20, low: this.position.y - 20 };
            var dfds = this.dfds;            
            var dfd = $.Deferred();
            var image = new Image();
            image.onload = function () {
                that.el.append($(this));
                that.el.append("<span class='name'>" + that.name + "</span>");
                that.container.append(that.el);
                dfd.resolve();
            };
            image.src = options.src;

           
            $.when(dfd.promise()).then(function () {
                that.animationQueue.resolve();
            });

            this.animationQueue.done(function () {
                that.animate();
                that.el.promise().done(function () {
                    setTimeout(function () {
                        that.personIcon.start();
                        that.bindEvents();
                    }, that.interVal);
                });
            });
        },
        animate: function () {
            this.el.effect("bounce", {}, this.duration);
        },
        bindEvents: function () {
            var that = this;
            this.el.hover(function (e) {
                that.personIcon.suspend();
               that.el.append(that.description);
            }, function (e) {
                that.personIcon.restart();
                that.el.remove(".description");
            });
        }
    }),

    PersonIcon = inherits(Icon, {
        init: function (options) {
            var that = this;
            this.el = $el = $("<image>");
            this.position = options.position || this.position;
            this.els = [];
            this.duration = options.duration || this.duration;
            this.interVal = options.interval || this.interVal;
            
            //we use that to promise that all images can be loaded normally
            var dfds = this.dfds;
            $.each(options.srcs, function (index, src) {
                var dfd = $.Deferred();
                var image = new Image();
                image.onload = function () {
                    that.els.push($(this));
                    dfd.resolve();
                };
                image.src = src;
                dfds.push(dfd.promise());
            });
            $.when.apply(this, dfds).then(function () {
                that.animationQueue.resolve();
            });

            this.animationQueue.done(function () {
                setInterval(function animation() {
                    that.animate.apply(that);
                }, that.interVal);
            });

        },
        animate: function () {
            var that = this;
            var elements = that.els;
            var len = elements.length;
            var frame = ~~(that.duration / len);
            var temHanler = setInterval(function animation() {
                var state = that.state;

                if (state.suspend) {
                    clearInterval(temHanler);
                } else {

                    state.index = state.index % len;
                    that.container.empty().append(elements[state.index]);
                    if (state.index === len - 1) {
                        state.direction = direction.endToStart;
                    }
                    if (state.direction === direction.endToStart && state.index === 0) {
                        state.direction = direction.startToEnd;
                        clearInterval(temHanler);
                    }
                    if (state.direction === direction.endToStart) {
                        state.index = --state.index;
                    } else {
                        state.index = ++state.index;
                    }
                }
            }, frame);
        },
        start: function () {
            var that = this;
            this.container.empty().append(this.els[0]);
            this.container.effect("bounce", { }, 500);
            this.container.promise().done(function () {
                that.state.suspend = false;
            });           
                    
        }
    });


    /*
    *Start the unit test part for this project
    * 
    */
    $(document).ready(function () {
        var countryArray = [
            {
                name: "英国",
                src: "images/map.png",
                person:{srcs: ['images/1.png', 'images/2.png', 'images/3.png', 'images/4.png']},
                position: {
                    x: 200,
                    y: 200
                },
                description: "<div class='description'><img src='images/korea.png'/></div>"
            },
            {
                name: "美国",
                src: "images/map.png",
                person: {
                    srcs: ['images/1.png', 'images/2.png', 'images/3.png', 'images/4.png'],
                    duration: 1000,
                    interval: 3000
                },
                position: {
                    x: 500,
                    y: 300
                },
                
            },
            {
                name: "韩国",
                src: "images/map.png",
                person: { srcs: ['images/1.png', 'images/2.png', 'images/3.png', 'images/4.png'] },
                position: {
                    x: 760,
                    y: 270
                },
                description: "<div class='description'><img src='images/korea.png'/></div>"
            },
        ];
        var $container = $("#map");       
        $.each(countryArray, function (index, element) {
            var country = new CountryIcon(element);
            country.container = $container;
        });

    });

})(jQuery);