﻿(function ($) {

    var ctor = function () { };

    var inherits = function (parent, protoProps, staticProps) {
        var child;

        if (protoProps && protoProps.hasOwnProperty('constructor')) {
            child = protoProps.constructor;
        } else {
            child = function () { parent.apply(this, arguments); };
        }

        $.extend(child, parent);

        ctor.prototype = parent.prototype;
        child.prototype = new ctor();

        if (protoProps) $.extend(child.prototype, protoProps);

        if (staticProps) $.extend(child, staticProps);

        child.prototype.constructor = child;

        child.__super__ = parent.prototype;

        return child;
    };

    var extend = function (protoProps, classProps) {
        var child = inherits(this, protoProps, classProps);
        child.extend = this.extend;
        return child;
    };

    /*
    *Start the main function of this project
    * 
    */

    function fixPNG(myImage) {
        var version = $.browser.version;
        var defaultWidth = 256, defaultHeight = 186;
        var width = myImage.width || defaultWidth;
        var height = myImage.height || defaultHeight;

        if ((version >= 5.5) && (version < 7) && (document.body.filters)) {

            var imgID = (myImage.id) ? "id='" + myImage.id + "' " : ""

            var imgClass = (myImage.className) ? "class='" + myImage.className + "' " : ""

            var imgTitle = (myImage.title) ?

                      "title='" + myImage.title + "' " : "title='" + myImage.alt + "' "

            var imgStyle = "display:inline-block;" + myImage.style.cssText

            var strNewHTML = "<span " + imgID + imgClass + imgTitle

                          + " style=\"" + "width:" + width

                          + "px; height:" + height

                          + "px;" + imgStyle + ";"

                          + "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader"

                          + "(src=\'" + myImage.src + "\', sizingMethod='scale');\"></span>"

            myImage.outerHTML = strNewHTML

        }

    }

    /*
    * Declare CountryIcon class
    */
    var direction = {
        startToEnd: 0,
        endToStart: 1
    }

    /*This function is the super class of all icons*/
    var Icon = function () {
        this.el = null;
        this.name = "";
        this.customClass = "";
        this.state = {
            index: 0,
            direction: direction.startToEnd,
            suspend: true
        };
        //will store all possible animation
        this.animationQueue = $.Deferred();
        //Indicate the position
        this.position = {
            x: 0,
            y: 0
        };
        //which element will contain this icon
        this.container = null;
        this.dfds = [];// store the function promises to ensure that all images can be loaded safetly
        //the time which the animation will examination
        this.duration = 250;
        //how much time will replay this animation
        this.interVal = 2000;

        //initialize this object
        this.init.apply(this, arguments);

        //Used to suspend the current animation
        this.suspend = function () {
            this.state.suspend = true;
        }
        //Restart the animation
        this.restart = function () {
            this.state.suspend = false;
        }
    },

    /*This is the icon of country*/
    CountryIcon = inherits(Icon, {
        init: function (options) {

            var that = this;//store itself used to clourse 

            /*This section for intializing the object*/

            //when mouse hover will show it
            this.description = "" || options.description;
            //used to mark the special for style
            this.customClass = options.customClass || this.customClass;
            //It will used to show the country name on map
            this.name = options.name;

            this.container = options.container || this.container;
            this.el = $("<div class='country'>");
            this.el.addClass(this.customClass);
            this.position = options.position || this.position;
            this.el.css({ "left": this.position.x, "top": this.position.y });
            this.personIcon = new PersonIcon(options.person);
            this.personIcon.container = this.el;
            this.duration = 150;
            //Define the boundary of the animation
            this.boundary = { high: this.position.y + 20, low: this.position.y - 20 };
            var dfds = this.dfds;
            var dfd = $.Deferred();
            var image = new Image();
            image.onload = function () {
                that.el.append($(this));
                that.el.append("<span class='name'>" + that.name + "</span>");
                that.container.append(that.el);
                dfd.resolve();
            };
            image.src = options.src;


            $.when(dfd.promise()).then(function () {
                that.animationQueue.resolve();
            });

            this.animationQueue.done(function () {
                that.animate();
                that.el.promise().done(function () {
                    setTimeout(function () {
                        that.personIcon.start();
                        that.bindEvents();
                    }, that.interVal);
                });
            });
        },
        animate: function () {
            this.el.effect("bounce", {}, this.duration);
        },
        bindEvents: function () {
            var that = this;
            var $description = $("<div class='description'><img/><p>粉丝数：<span class='fans'></span></p></div>");
            fixPNG($("img", $description).attr('src', that.description.image)[0]);
            $description.css({ 'top': that.description.position.y || 0, 'left': that.description.position.x || 0 })
            this.el.hover(function (e) {
                that.personIcon.suspend();
                $(".description").remove();
                that.el.append($description);
            }, function (e) {
                that.personIcon.restart();
                that.el.remove(".description");
            });
        }
    }),

    PersonIcon = inherits(Icon, {
        init: function (options) {
            var that = this;
            this.el = $el = $("<image>");
            this.position = options.position || this.position;
            this.els = [];
            this.duration = options.duration || this.duration;
            this.interVal = options.interval || this.interVal;
            this.container = options.container || this.container;

            //we use that to promise that all images can be loaded normally
            var dfds = this.dfds;
            $.each(options.srcs, function (index, src) {
                var dfd = $.Deferred();
                var image = new Image();
                image.onload = function () {
                    that.els.push($(this));
                    dfd.resolve();
                };
                image.src = src;
                dfds.push(dfd.promise());
            });
            $.when.apply(this, dfds).then(function () {
                that.animationQueue.resolve();
            });

            this.animationQueue.done(function () {
                setInterval(function animation() {
                    that.animate.apply(that);
                }, that.interVal);
            });

        },
        animate: function () {
            var that = this;
            var elements = that.els;
            var len = elements.length;
            var frame = ~~(that.duration / len);
            var temHanler = setInterval(function animation() {
                var state = that.state;

                if (state.suspend) {
                    clearInterval(temHanler);
                } else {
                    state.index = state.index % len;
                    that.container.empty().append(elements[state.index]);
                    if (state.index === len - 1) {
                        state.direction = direction.endToStart;
                        clearInterval(temHanler);
                    }
                    state.index++;
                }
            }, frame);
        },
        start: function () {
            var that = this;
            this.container.empty().append(this.els[0]);
            this.container.effect("bounce", {}, 500);
            this.container.promise().done(function () {
                that.state.suspend = false;
            });

        }
    });

    var TitleIcon = inherits(Icon, {
        init: function (options) {
            var that = this;
            this.el = $el = $("<image>");
            this.position = options.position || this.position;
            this.els = [];
            this.duration = options.duration || this.duration;
            this.interVal = options.interval || this.interVal;
            this.container = options.container || this.container;
            //we use that to promise that all images can be loaded normally
            var dfds = this.dfds;
            $.each(options.srcs, function (index, src) {
                var dfd = $.Deferred();
                var image = new Image();
                image.onload = function () {
                    that.els.push($(this));
                    dfd.resolve();
                };
                image.src = src;
                dfds.push(dfd.promise());
            });
            $.when.apply(this, dfds).then(function () {
                that.animationQueue.resolve();
            });

            this.animationQueue.done(function () {
                that.createElements();
                that.animate();
            });

        },
        animate: function () {
            $(".box", this.container).animate({ left: '657px' }, this.interVal);
            $(".ball", this.container).animate({ top: '10px' }, 800);
        },
        createElements: function () {
            var element = $("<div/>");
            var startElem = element.clone().append(this.els[0]).attr("class", "start"),
                bottomElem = element.clone().append(this.els[1]).attr("class", "bottom"),
                manElem = element.clone().append(this.els[2]).attr("class", "man"),
                boxElem = element.clone().append(this.els[3]).attr("class", "box");
            ballElem = element.clone().append(this.els[4]).attr("class", "ball");
            this.container.append(startElem).append(bottomElem).append(manElem).append(boxElem).append(ballElem);
        }
    });

    /*
    *Start the unit test part for this project
    * 
    */
    $(document).ready(function () {
        var $countryContainer = $("#map"),
            $beautyContainer = $("#beatutyBox"),
            imageRoot = "images/";
            countryArray = [
            {
                name: "意大利",
                customClass: 'italy',
                src: imageRoot + "map.png",
                person: {
                    srcs: [imageRoot + 'Italy_1.png', imageRoot + 'Italy_2.png', imageRoot + 'Italy_3.png', imageRoot + 'Italy_1.png'],
                    duration: 800,
                    interval: 3000
                },
                position: {
                    x: 480,
                    y: 282
                },
                container: $countryContainer,
                description: { image: imageRoot + 'Italy.png', position: { x: '-20px', y: '60px' } }
            },
            {
                name: "美国",
                customClass: 'america',
                src: imageRoot + "map.png",
                person: {
                    srcs: [imageRoot + 'America_1.png', imageRoot + 'America_2.png', imageRoot + 'America_3.png', imageRoot + 'America_4.png', imageRoot + 'America_1.png'],
                    duration: 1000,
                    interval: 2000
                },
                position: {
                    x: 215,
                    y: 255
                },
                container: $countryContainer,
                description: { image: imageRoot + 'America.png', position: { x: '30px', y: '80px' } }

            },
            {
                name: "韩国",
                customClass: 'korea',
                src: imageRoot + "map.png",
                person: {
                    srcs: [imageRoot + 'korea_1.png', imageRoot + 'korea_2.png', imageRoot + 'korea_3.png', imageRoot + 'korea_1.png'],
                    duration: 700,
                    interval: 2500
                },
                position: {
                    x: 760,
                    y: 270
                },
                container: $countryContainer,
                description: { image: imageRoot + "korea.png", position: { x: '-220px', y: '80px' } }
            },
            {
                name: "英国",
                customClass: 'england',
                src: imageRoot + "map.png",
                person: {
                    srcs: [imageRoot + 'England_1.png', imageRoot + 'England_2.png', imageRoot + 'England_3.png', imageRoot + 'England_4.png', imageRoot + 'England_1.png'],
                    duration: 600
                },
                position: {
                    x: 434,
                    y: 173
                },
                container: $countryContainer,
                description: { image: imageRoot + "England.png", position: { x: '-20px', y: '80px' } }
            },
            {
                name: "法国",
                customClass: 'france',
                src: imageRoot + "map.png",
                person: {
                    srcs: [imageRoot + 'France_1.png', imageRoot + 'France_2.png', imageRoot + 'France_3.png', imageRoot + 'France_1.png'],
                    duration: 600,
                    interval: 2300
                },
                position: {
                    x: 530,
                    y: 196
                },
                container: $countryContainer,
                description: { image: imageRoot + "France.png", position: { x: '-40px', y: '80px' } }
            },
             {
                 name: "西班牙",
                 customClass: 'spain',
                 src: imageRoot + "map.png",
                 person: {
                     srcs: [imageRoot + 'Spain_1.png', imageRoot + 'Spain_2.png', imageRoot + 'Spain_3.png', imageRoot + 'Spain_1.png'],
                     duration: 600,
                     interval: 3200
                 },
                 position: {
                     x: 385,
                     y: 250
                 },
                 container: $countryContainer,
                 description: { image: imageRoot + "Spain.png", position: { x: '-40px', y: '80px' } }
             }
            ];

        var titleOptions = {
            srcs: [imageRoot + "start.png", imageRoot + "bottom.png", imageRoot + "man.png", imageRoot + "box.png", imageRoot + "ball.png"],
            container: $beautyContainer,
            interval: 1000
        };
        var country;

        $.each(countryArray, function (index, element) {
            country = new CountryIcon(element);
        });

        var titleIcon = new TitleIcon(titleOptions);


    });

})(jQuery);